import 'package:json_annotation/json_annotation.dart';

part 'mc_scanning_data.g.dart';

@JsonSerializable(
    createFactory: true, includeIfNull: false, createToJson: false)
class MedicalCertificateResult {
  /// Defines a result of the Medical Certificate scanning flow. */ class MedicalCertificateRecognizerResult {

  /// Defines whether recognition was successful
  bool recognitionSuccessful;

  /// Vector of found checkboxes

  List<MedicalCertificateInfoBox> checkboxes;

  /// Vector of validated dates

  List<DateRecord> dates;

  /// Information about the patient

  List<McPatientInfoField> patientInfoFields;

  /// Type of MC form

  McFormType mcFormType = McFormType.Unknown;

  Uri? croppedDocumentURI;

  MedicalCertificateResult(this.recognitionSuccessful, this.checkboxes,
      this.dates, this.patientInfoFields, this.mcFormType);

  factory MedicalCertificateResult.fromJson(Map<String, dynamic> json) =>
      _$MedicalCertificateResultFromJson(json);
}

/// Structure to contain full information about found box
@JsonSerializable(
    createFactory: true, includeIfNull: false, createToJson: false)
class MedicalCertificateInfoBox {
  /// Box subtype

  McInfoBoxSubtype subType;

  /// If box is a checkbox, this field shows whether the checkbox is checked. Not used otherwise

  bool hasContents;

  /// Contents validation confidence value

  double contentsValidationConfidenceValue;

  MedicalCertificateInfoBox(
      this.subType, this.hasContents, this.contentsValidationConfidenceValue);

  factory MedicalCertificateInfoBox.fromJson(Map<String, dynamic> json) =>
      _$MedicalCertificateInfoBoxFromJson(json);
}

/// Box initial type
enum McInfoBoxType { McRectPatientInfoBox, McRectCheckbox }

/// Box detailed subtype
enum McInfoBoxSubtype {
  McBoxUnknown,
  McBoxPatientInfo,
  McBoxWorkAccident,
  McBoxAssignedToAccidentInsuranceDoctor,
  McBoxInitialCertificate,
  McBoxRenewedCertificate,
  McBoxInsuredPayCase,
  McBoxFinalCertificate,
  McBoxRequiresCareYes,
  McBoxRequiresCareNo,
  McBoxAccidentYes,
  McBoxAccidentNo,
  McBoxOtherAccident,
  McBoxEntitlementToContinuedPaymentYes,
  McBoxEntitlementToContinuedPaymentNo,
  McBoxSickPayWasClaimedNo,
  McBoxSickPayWasClaimedYes,
  McBoxSingleParentNo,
  McBoxSingleParentYes
}

/// Structure to contain date record information
@JsonSerializable(
    createFactory: true, includeIfNull: false, createToJson: false)
class DateRecord {
  /// Date string
  String dateString;

  /// Date type
  DateRecordType type;

  /// Recognition confidence value
  double recognitionConfidenceValue;

  /// Validation confidence value
  double validationConfidenceValue;

  DateRecord(this.dateString, this.type, this.recognitionConfidenceValue,
      this.validationConfidenceValue);

  factory DateRecord.fromJson(Map<String, dynamic> json) =>
      _$DateRecordFromJson(json);
}

/// Type of a date record
enum DateRecordType {
  DateRecordIncapableOfWorkSince,
  DateRecordIncapableOfWorkUntil,
  DateRecordDiagnosedOn,
  DateRecordDocumentDate,
  DateRecordBirthDate,
  DateRecordChildNeedsCareFrom,
  DateRecordChildNeedsCareUntil,
  DateRecordUndefined
}

/// Represents an item of the patient info
@JsonSerializable(
    createFactory: true, includeIfNull: false, createToJson: false)
class McPatientInfoField {
  McPatientInfoFieldType patientInfoFieldType;
  String value;

  McPatientInfoField(this.patientInfoFieldType, this.value);

  factory McPatientInfoField.fromJson(Map<String, dynamic> json) =>
      _$McPatientInfoFieldFromJson(json);
}

/// Form type of the Medical certificate
enum McFormType {
  Form_1a,
  Form_1b,
  Form_1c,
  Form_1d,
  Form_21a,
  Form_21a_back,
  Form_1b_Custom,
  Unknown
}

/// Type of the item of the patient info field
enum McPatientInfoFieldType {
  InsuranceProvider,
  FirstName,
  LastName,
  AddressString1,
  AddressString2,
  Diagnose,
  HealthInsuranceNumber,
  InsuredPersonNumber,
  Status,
  PlaceOfOperationNumber,
  DoctorNumber,
  Undefined
}
